/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.container;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;

import org.exoplatform.container.configuration.ConfigurationManager;
import org.exoplatform.container.configuration.ConfigurationManagerImpl;
import org.exoplatform.container.jmx.MX4JComponentAdapterFactory;
import org.exoplatform.container.monitor.jvm.J2EEServerInfo;
import org.exoplatform.container.util.ContainerUtil;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov
 *         </a>
 * @version $Id: StandaloneContainer.java 3510 2005-12-29 17:32:57Z geaz $
 * 
 * Singletone, context independent Exo Container with one configuration entry point.
 * The configuration is set as follows:
 * - client calls setConfigurationURL() or setConfigurationPath method BEFORE getInstance()
 * - otherwise container in instantiation time looks for configuration.xml file in the "home" directory.
 *   the home directory it is AS server home in a case of AS env or just current directory (from where JVM is started) for standalone.
 *   See      
 */

public class StandaloneContainer extends ExoContainer {

  private static StandaloneContainer container;

  private static URL configurationURL = null;

  private SessionManager smanager_;

  private ConfigurationManagerImpl configurationManager;

  private StandaloneContainer() {
    super(new MX4JComponentAdapterFactory(), null);
    configurationManager = new ConfigurationManagerImpl(null);
    this.registerComponentInstance(ConfigurationManager.class,
        configurationManager);
    registerComponentImplementation(SessionManagerImpl.class);
  }

  public static StandaloneContainer getInstance() throws Exception {
    if (container == null) {
      container = new StandaloneContainer();
      URL config = container.initConfig(
        Thread.currentThread().getContextClassLoader().getResource("conf/standalone/configuration.xml"),
        configurationURL);
      
        // System. out.println("-- StandaloneContainer: used url: " + config);

      if(config != null) {
        container.populate(config);  
      } else {
        throw new Exception("*** No StandaloneContainer config found. Check if conf/standalone/configuration.xml exists !");
      }
      
      Iterator c = container.getComponentAdapters().iterator();
      container.start();
      System.setProperty("exo.standalone-container", StandaloneContainer.class.getName());
    }
    return container;
  }

  public SessionContainer createSessionContainer(String id) {
    SessionContainer scontainer = getSessionManager().getSessionContainer(id);
    if(scontainer != null)   getSessionManager().removeSessionContainer(id);
    scontainer = new SessionContainer(id, null);
    getSessionManager().addSessionContainer(scontainer);
    SessionContainer.setInstance(scontainer);
    return scontainer;
  }
  
  public void removeSessionContainer(String sessionID) {
    getSessionManager().removeSessionContainer(sessionID);
  }
  
  public static void setConfigurationURL(String url) throws MalformedURLException {
    if((url == null) || (url.length() == 0))
      return;
    configurationURL = new URL(url);
  }
  
  public static void setConfigurationPath(String url) throws MalformedURLException {
    if((url == null) || (url.length() == 0))
      return;
    configurationURL = (new File(url)).getAbsoluteFile().toURL();
  }
  
  public MBeanServer getMBeanServer() {
    return MBeanServerFactory.createMBeanServer("exojcrmx");
  }

  public URL getConfigurationURL() {
    return configurationURL;
  }
  
  // -------------- Helpers ----------
  
  private SessionManager  getSessionManager()  {
    if(smanager_ == null)
      smanager_ = (SessionManager) this.getComponentInstanceOfType(SessionManager.class);
    return smanager_;
  }

  private URL initConfig(URL base, URL additional) {
    URL result = base;
    if (fileExists(additional)) {
      result = additional;
    }
    else
    {
      try {
        URL local = new URL("file:/" + (new J2EEServerInfo()).getServerHome() + "/configuration.xml");
         //System.out.println("-- StandaloneContainer: server home conf: " + local.toString());
        if (fileExists(local))
          result = local;
      } catch(Exception e) {}
    }
    this.configurationURL = result;
    return result;
  }
  
  private boolean fileExists(URL url) {
    try {
      // System.out.println("-- StandaloneContainer.checkURL: " + url + " exists: " + (new File(url.toURI())).exists());
      return (new File(url.toURI())).exists();
    } catch (Exception e) {
      // System.out.println("-- StandaloneContainer.checkURL: exception: " + e);
      return false;
    }
  }

  private void populate(URL conf) throws Exception {
    configurationManager.addConfiguration(conf);
    configurationManager.processRemoveConfiguration();
    ContainerUtil.addComponents(this, configurationManager);
  }

  private void populateGroovy(URL conf) throws Exception {
    configurationManager.addConfiguration(conf);
    configurationManager.processRemoveConfiguration();
  //ContainerUtil.populateGroovy(this, configurationManager);
  }

  /**
   * @deprecated 
   */
  public ConfigurationManagerImpl getConfigurationManager() {
    return configurationManager;
  }
}