/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.templates.velocity.impl;

import java.io.Reader;
import java.util.*;
import org.apache.commons.collections.ExtendedProperties;
import org.apache.commons.logging.Log;
import org.apache.velocity.Template;
import org.apache.velocity.exception.ParseErrorException;
import org.apache.velocity.exception.ResourceNotFoundException;
import org.apache.velocity.runtime.RuntimeConstants;
import org.apache.velocity.runtime.VelocimacroFactory;
import org.apache.velocity.runtime.directive.Directive;
import org.apache.velocity.runtime.parser.ParseException;
import org.apache.velocity.runtime.parser.Parser;
import org.apache.velocity.runtime.parser.node.SimpleNode;
import org.apache.velocity.runtime.resource.ContentResource;
import org.apache.velocity.runtime.resource.ResourceManager;
import org.apache.velocity.util.introspection.Introspector;
import org.apache.velocity.util.introspection.Uberspect;
import org.apache.velocity.util.introspection.UberspectLoggable;
import org.exoplatform.commons.utils.ExoProperties;
import org.exoplatform.container.ExoContainer;
import org.exoplatform.container.component.ComponentLifecycle;
import org.exoplatform.container.component.ComponentPlugin;
import org.exoplatform.container.xml.InitParams;
import org.exoplatform.services.log.LogService;
import org.exoplatform.services.templates.velocity.StaticResourceLoader;
import org.exoplatform.services.templates.velocity.VelocityService;

/**
 * Created y the eXo platform team
 * User: Tuan Nguyen
 * Date: march 2, 2005
 */
public class VelocityServiceImpl implements RuntimeConstants, VelocityService, ComponentLifecycle {
  private  VelocimacroFactory vmFactory = null;
  private ThreadLocal threadParser_ = new ThreadLocal() ;
  
  private  Hashtable runtimeDirectives_ = new Hashtable(); 
  private  ExtendedProperties properties_ = new ExtendedProperties();
  
  private ResourceManagerImpl resourceManager_ = null ;
  private Introspector introspector = null;
  private Map applicationAttributes = null;
  
  private Uberspect uberSpect_;
  private Log log_ ;
  
  public VelocityServiceImpl(InitParams params,
                             LogService lservice, 
                             Uberspect uberSpect, 
                             ResourceManagerImpl resourceManager)  throws Exception {
    log_ = lservice.getLog(VelocityService.class) ;
    introspector = new Introspector( this );
    applicationAttributes = new HashMap() ;
    uberSpect_ = uberSpect ;
    uberSpect_.init(); 
    if (uberSpect instanceof UberspectLoggable)  {
      ((UberspectLoggable)uberSpect).setRuntimeLogger(this);
    }
    uberSpect.init();

    introspector = new Introspector( this );
    List directives = params.getValuesParam("velocity.directives").getValues() ;
    for(int i = 0; i < directives.size(); i++ ) {
      String directiveClass = (String) directives.get(i) ;
      Directive directive = (Directive)Class.forName( directiveClass ).newInstance();
      runtimeDirectives_.put(directive.getName(), directive);
    }
    ExoProperties exoProps = params.getPropertiesParam("velocity.properties").getProperties() ;
    Iterator i = exoProps.entrySet().iterator() ;
    while(i.hasNext()) {
      Map.Entry entry = (Map.Entry) i.next() ;
      properties_.addProperty((String) entry.getKey(), (String) entry.getValue()) ;
    }
    resourceManager_ = resourceManager ;
    resourceManager_.initialize(this)  ;
  } 
  
  public void initComponent(ExoContainer container) throws Exception {      
    vmFactory = new VelocimacroFactory( this );
    vmFactory.initVelocimacro() ;
  }

  public void startComponent(ExoContainer container) throws Exception {  }

  public void stopComponent(ExoContainer container) throws Exception {  }

  public void destroyComponent(ExoContainer container) throws Exception { }
  
  public synchronized void init() throws Exception {
    throw new RuntimeException("You cannot use this method , use configuration.xml") ; 
  }
  
  
  public void setProperty(String key, Object value){
    throw new RuntimeException("You cannot reset the properties , use configuration.xml") ; 
  }        
  
  public void setConfiguration( ExtendedProperties configuration) {
    throw new RuntimeException("You cannot use this method , use configuration.xml") ; 
  }
  
  public void addProperty(String key, Object value) {
    throw new RuntimeException("You cannot use this method , use configuration.xml") ; 
  }
  
  public void clearProperty(String key) {
    throw new RuntimeException("You cannot use this method , use configuration.xml") ;          
  }
  
  public Object getProperty( String key ) { return properties_.getProperty( key ); }
  
  public void init(Properties p) throws Exception  {
    throw new RuntimeException("You cannot use this method , use init() and configuration.xml") ;  
  }
  
  public void init(String configurationFile) throws Exception  {
    throw new RuntimeException("You cannot use this method , use init() and configuration.xml") ;  
  }
  
  public Parser getParser() {
    Parser parser = (Parser)threadParser_.get() ;
    if(parser == null) {
      parser = new Parser( this );
      parser.setDirectives(runtimeDirectives_);
      threadParser_.set(parser) ;
    }
    return parser;
  }
  
  public SimpleNode parse( Reader reader, String templateName ) throws ParseException {
    return parse( reader, templateName, true );
  }
  
  public SimpleNode parse( Reader reader, String templateName, boolean dumpNamespace ) throws ParseException  {
    SimpleNode ast = null;
    Parser parser = getParser() ;
    if ( dumpNamespace ) {
      dumpVMNamespace( templateName );
    }
    ast = parser.parse( reader, templateName );
    return ast;
  }
  
  public Template getTemplate(String name)  throws ResourceNotFoundException, 
                                            ParseErrorException, Exception  {
    return getTemplate( name, getString( INPUT_ENCODING, ENCODING_DEFAULT) );
  }
  
  public Template getTemplate(String name, String  encoding) throws ResourceNotFoundException, 
                                                             ParseErrorException, Exception  {
    return (Template) resourceManager_.getResource(name, ResourceManager.RESOURCE_TEMPLATE, encoding);
  }
  
  public ContentResource getContent(String name)  throws ResourceNotFoundException, 
                                                  ParseErrorException, Exception {
    return getContent( name,  getString( INPUT_ENCODING, ENCODING_DEFAULT));
  }
  
  public ContentResource getContent( String name, String encoding ) throws ResourceNotFoundException, 
                                                                    ParseErrorException, Exception {
    return (ContentResource) resourceManager_.getResource(name,ResourceManager.RESOURCE_CONTENT, encoding );
  }
  
  public String getLoaderNameForResource( String resourceName ) {
    return resourceManager_.getLoaderNameForResource( resourceName );
  }
  
  public String getString( String key, String defaultValue) {
    return properties_.getString(key, defaultValue);
  }
  
  public Directive getVelocimacro( String vmName, String templateName  ) {
    return vmFactory.getVelocimacro( vmName, templateName );
  }
  
  public boolean addVelocimacro( String name, String macro, String argArray[], String sourceTemplate ) {
    return vmFactory.addVelocimacro(  name, macro,  argArray,  sourceTemplate );
  }
  
  public boolean isVelocimacro( String vmName, String templateName )  {
    return vmFactory.isVelocimacro( vmName, templateName );
  }
  
  public boolean dumpVMNamespace( String namespace ) {
    return vmFactory.dumpVMNamespace( namespace );
  }

  public String getString(String key)  { return properties_.getString( key ); }
  
  public int getInt( String key ) {  return properties_.getInt( key ); }
 
  public int getInt( String key, int defaultValue )  {
    return properties_.getInt( key, defaultValue );
  }
  
  public boolean getBoolean( String key, boolean def ) {
    return properties_.getBoolean( key, def );
  }
  
  public ExtendedProperties getConfiguration() {  return properties_; }        
  
  public Introspector getIntrospector() {  return introspector; }
  
  public Object getApplicationAttribute( Object key) {
    return applicationAttributes.get( key );
  }
  
  public Object setApplicationAttribute( Object key, Object o ) {
    return applicationAttributes.put( key, o );
  }
  
  public Uberspect getUberspect() {   return uberSpect_; }
  
  public void warn(Object message) {
    if(message instanceof Throwable) {
      Throwable t = (Throwable) message ;
      if(t.getCause() != null) t = t.getCause() ;
      log_.warn("Error" , t);
    } else {
      log_.warn(message);
    }
  }
  
  public void info(Object message) {
    log_.info(message); 
  }
  
  public void error(Object message) { 
    if(message instanceof Throwable) {
      Throwable t = (Throwable) message ;
      if(t.getCause() != null) t = t.getCause() ;
      log_.error("Error" , t);
    } else {
      log_.error(message);
    }
  }
  
  public void debug(Object message) { log_.debug(message); }

  public void addPlugin(ComponentPlugin plugin) {
    resourceManager_.addPlugin(plugin)  ;
  }
  
  public StaticResourceLoader getStaticResourceLoader()  {
    return resourceManager_.getStaticResourceLoader() ;
  }
}