/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 ***************************************************************************/
package org.exoplatform.portal.faces.renderer;

import java.io.IOException;
import java.net.URL;
import java.util.List;
import java.util.Map;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.servlet.ServletContext;

import org.apache.velocity.Template;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.container.SessionContainer;
import org.exoplatform.faces.core.component.ComponentTemplate;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.renderer.html.VelocityRenderer;
import org.exoplatform.portal.faces.component.UIPortalComponent;
import org.exoplatform.portal.session.RequestInfo;
import org.exoplatform.services.portal.model.Component;
import org.exoplatform.services.templates.velocity.StaticResourceLoader;
import org.exoplatform.services.templates.velocity.VelocityService;
/**
 * Created by The eXo Platform SARL        .
 * Date: Mar 15, 2003
 * Time: 11:30:40 AM
 */
abstract public class BaseRenderer extends VelocityRenderer {
  
  public void decode(FacesContext context, UIComponent uiComponent) {
    RequestInfo rinfo = (RequestInfo) SessionContainer.getComponent(RequestInfo.class);
    String portalAction = rinfo.getPortalAction();
    if(portalAction != null) {
      UIPortalComponent basicComponent = (UIPortalComponent) uiComponent ;
      Map paramMap = context.getExternalContext().getRequestParameterMap() ;
      basicComponent.broadcast(new ExoActionEvent(uiComponent, portalAction, paramMap)) ;
    } else {
      decodeComponentAction(context, uiComponent) ;
    }
  }
  
  protected void decodeComponentAction(FacesContext context, UIComponent uiComponent) {
  }
  
  protected Template getTemplate(UIComponent component, FacesContext fcontext) throws Exception {
    VelocityService vservice =  
      (VelocityService) PortalContainer.getComponent(VelocityService.class) ;
    StaticResourceLoader srloader = vservice.getStaticResourceLoader() ;
    String name =  ((ComponentTemplate)component).getTemplatePath() ;
    try {
      Template template  = srloader.getTemplate(name) ;
      if(template != null)  return template ;
    } catch(Exception ex) { }
    
    URL url = null ;
    if(url == null) {
      ServletContext context = (ServletContext) fcontext.getExternalContext().getContext();
      String newPath = context.getRealPath("/velocity/") ;
      if(newPath != null)srloader.addPath(newPath) ;
      url = getResource(context, "/velocity/" + name) ;
      //S ystem.out.println("LOAD FROM PORTAL: " + url) ;
    }
    if(url == null){
      url = Thread.currentThread().getContextClassLoader().getResource(name);
      //S ystem.out.println("LOAD FROM CLASSPATH: " + url) ;
    }
    srloader.addTemplate(name, url) ;
    return vservice.getTemplate(name) ;
  }
  
  final public void encodeBegin(FacesContext context, UIComponent uiComponent) throws IOException {
    throw new IOException("use startRenderEditMode, startRenderEditMode, and renderViewMode") ;
  }
  
  final public void encodeEnd(FacesContext context, UIComponent uiComponent) throws IOException {
    throw new IOException("use startRenderEditMode, startRenderEditMode, and renderViewMode") ;
  }
  
  public void startRenderEditMode(FacesContext context, UIPortalComponent uiComponent) throws IOException {
    
  }
  
  public void renderViewMode(FacesContext context, UIPortalComponent uiComponent) throws IOException {
    throw new IOException("You need to  override this method") ;
  }
  
  public void endRenderEditMode(FacesContext context, UIPortalComponent uiComponent) throws IOException {
    
  }
  
  public String getComponentHeight(UIPortalComponent  uiComponent) {
    Component component =  uiComponent.getComponentModel() ;
    String height =  component.getHeight() ;
    if (height != null && height.length() > 0)   return height ;
    return "auto" ;
  }
  
  public String getComponentHeight(UIPortalComponent  uiComponent, String defaultHeight) {
    Component component =  uiComponent.getComponentModel() ;
    String height =  component.getHeight() ;
    if (height != null && height.length() > 0)   return height ;
    return defaultHeight ;
  }
  
  public String getComponentWidth(UIPortalComponent  uiComponent, String defaultWidth) {
    Component component =  uiComponent.getComponentModel() ;
    String width =  component.getWidth() ;
    if (width != null && width.length() > 0)   return width ;
    return defaultWidth ;
  }
  
  public void renderChildFloatVertical(UIPortalComponent component, FacesContext context) throws IOException {
    component.setFloat(UIPortalComponent.FLOAT_BOTTOM) ;
    component.encodeBegin(context) ;
    component.encodeChildren(context) ;
    component.encodeEnd(context) ;
  }
  
  public void renderChildFloatHorizontal(UIPortalComponent component, FacesContext context) throws IOException {
    component.setFloat(UIPortalComponent.FLOAT_RIGHT) ;
    component.encodeBegin(context) ;
    component.encodeChildren(context) ;
    component.encodeEnd(context) ;
  }
  
  public  void ensureOneChildIsRendered(List<UIComponent> children) {
    UIComponent select = null ;
    for(UIComponent uicomponent : children) {
      if(select != null) {
        uicomponent.setRendered(false) ;
      } else {
        if(uicomponent.isRendered())  select =  uicomponent ;
      }
    }
    if(select  == null) {
      select = children.get(0) ;
      select.setRendered(true) ;
    }
  }
}