/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.backup.component;

import java.io.File;
import java.util.List;

import javax.faces.context.FacesContext;
import javax.servlet.http.HttpServletResponse;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.UINode;
import org.exoplatform.faces.core.component.UIPageListIterator;
import org.exoplatform.faces.core.component.model.PageListDataHandler;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portlets.backup.component.model.ImportData;
import org.exoplatform.services.backup.ImportExportService;
import org.exoplatform.services.download.DownloadResource;
import org.exoplatform.services.download.DownloadService;
import org.exoplatform.services.download.FileDownloadResource;

/**
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIImportData.java,v 1.5 2004/09/21 00:16:11 tuan08 Exp $
 */
public class UIBackupData extends UIExoCommand {
  static String IMPORT_DATA = "importData" ;
  static String VIEW_DATA = "viewData" ;
  static String DOWNLOAD_DATA = "downloadData" ;
  static String DELETE_DATA = "deleteData" ;
  static String REFRESH_DATA = "refreshData" ;
  static String IMPORT_ALL_DATA = "importAllData" ;
  static String DOWNLOAD_ALL_DATA = "downloadAllData" ;
  
  static Parameter[] IMPORT_PARAMS = { new Parameter(ACTION , IMPORT_DATA )} ;
  static Parameter[] VIEW_PARAMS = { new Parameter(ACTION , VIEW_DATA )} ;  
  static Parameter[] DOWNLOAD_PARAMS = { new Parameter(ACTION , DOWNLOAD_DATA )} ;
  static Parameter[] DELETE_PARAMS = { new Parameter(ACTION , DELETE_DATA )} ;
  static Parameter[] REFRESH_PARAMS = { new Parameter(ACTION , REFRESH_DATA )} ;
  static Parameter[] IMPORT_ALL_PARAMS  =   { new Parameter(ACTION , IMPORT_ALL_DATA) } ;
  static Parameter[] DOWNLOAD_ALL_PARAMS  =   { new Parameter(ACTION , DOWNLOAD_ALL_DATA) } ;
  
  protected ImportExportService service_ ;
  protected String directory_ ;  
  protected int pageSize = 25;
  protected boolean adminRole_ = false ;
  protected UIPageListIterator uiPageIterator_ ;

  public UIBackupData(ImportExportService service) {
  	setRendererType("VelocityRenderer") ;
  	service_ = service ;
    uiPageIterator_ = new UIPageListIterator(new PageListDataHandler()) ; 
    getChildren().add( uiPageIterator_) ;
  }
  
  public Parameter[] getImportParams() { return IMPORT_PARAMS ; }
  public Parameter[] getViewParams() { return VIEW_PARAMS ; }
  public Parameter[] getDownloadParams() { return  DOWNLOAD_PARAMS ; }
  public Parameter[] getDeleteParams() { return  DELETE_PARAMS ; }
  public Parameter[] getRefreshParams() { return REFRESH_PARAMS ; }  
  public Parameter[] getImportAllParams() { return IMPORT_ALL_PARAMS ; }
  public Parameter[] getDownloadAllParams() { return DOWNLOAD_ALL_PARAMS ; }
  
  public boolean hasAdminRole() { return adminRole_ ; }
  public void    setAdminRole(boolean b) { adminRole_ = b ; }
  
  ImportExportService getBackupService() { return service_ ; } 
  
  public String getDirectory() { return directory_ ; }
  public void   setDirectory(String s) {
    directory_ = s ;
    refresh();
  }
  
  public List getImportDatas() {
    return uiPageIterator_.getPageListDataHandler().getDatas();     
  }  
  
  public void refresh(){
    try{
      uiPageIterator_.setPageList( new ZipFilePageList( directory_, pageSize));
    }catch( Exception exp){
      exp.printStackTrace();
    }
  }
  
  static public class RefreshActionListener extends ExoActionListener  {
  	public void execute(ExoActionEvent event) throws Exception {
      UIBackupData ui = (UIBackupData) event.getComponent() ;
  		ui.refresh();
    }
  }
  
  
  static public class ViewDataActionListener extends ExoActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UIBackupData ui = (UIBackupData) event.getComponent() ;
      UINode parent = (UINode) ui.getParent() ;
      UIData uiData = (UIData)  parent.getChildComponentOfType(UIData.class) ;
      ImportData data = ui.findImportData(event.getParameter(OBJECTID));
      if( data == null) return;
      uiData.setImportDataModel(data) ;
      parent.setRenderedComponent(uiData.getId()) ; 
    }
  }
  
  static public class DownloadDataActionListener extends ExoActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UIBackupData ui = (UIBackupData) event.getComponent() ;
      String dataName = event.getParameter(OBJECTID);
      ImportData data = ui.findImportData(  dataName);
      if( data == null) return;
      File in = new File(data.getResourceURL());
      DownloadService dservice =
        ( DownloadService) PortalContainer.getComponent(DownloadService.class);
      DownloadResource dresource = new FileDownloadResource(in.getAbsolutePath(), "application/zip") ;
      String downloadId = dservice.addDownloadResource(dresource) ;
      dresource.setDownloadName( dataName);
      String downloadLink = dservice.getDownloadLink(downloadId) ;
      FacesContext ctx = ui.getFacesContext();             
      HttpServletResponse response =(HttpServletResponse)ctx.getExternalContext().getResponse();      
      response.sendRedirect(downloadLink);
      ctx.responseComplete();     
    }    
  }
  
  static public class DeleteDataActionListener extends ExoActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UIBackupData ui = (UIBackupData) event.getComponent() ;      
      ImportData data = ui.findImportData(event.getParameter(OBJECTID));     
      if(data == null) return;
      String path  = data.getResourceURL();
      List<ImportData> list = ui.getImportDatas();
      list.remove(data);
      data.close();
      data = null;     
      System.gc();
      File file = new File(path);       
      file.delete();      
      ui.refresh();
    }   
  }
  
  private ImportData findImportData(String dataName) {
    if( dataName == null || dataName.trim().equals("")) return null;
    List<ImportData> list = getImportDatas();
    for(ImportData data : list) {        
      if(data.getName().equals(dataName)) return data;          
    }   
    return null;
  }
}