/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.backup.component;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.util.zip.ZipOutputStream;

import javax.faces.context.FacesContext;
import javax.servlet.http.HttpServletResponse;

import org.exoplatform.commons.utils.io.ZipUtil;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.faces.core.event.CheckRoleInterceptor;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.services.backup.ImportExportService;
import org.exoplatform.services.backup.Metadata;
import org.exoplatform.services.download.DownloadResource;
import org.exoplatform.services.download.DownloadService;
import org.exoplatform.services.download.InputStreamDownloadResource;
import org.exoplatform.xml.object.XMLObject;
/**
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIImportUserData.java,v 1.8 2004/09/21 00:16:11 tuan08 Exp $
 */
public class UIBackupUserData extends UIBackupData {
  
  public UIBackupUserData(ImportExportService service) {
    super(service) ;
    setId("UIImportUserData") ;
    setDirectory( service_.getUserDataDirectory()) ;    
    
    addActionListener(RefreshActionListener.class, REFRESH_DATA) ;
    addActionListener(ViewDataActionListener.class, VIEW_DATA) ;
    addActionListener(DownloadDataActionListener.class, DOWNLOAD_DATA) ;
    addActionListener(DeleteDataActionListener.class, DELETE_DATA) ;
    addActionListener(ImportUserActionListener.class, IMPORT_DATA) ;
    addActionListener(ImportAllActionListener.class, IMPORT_ALL_DATA) ;
    addActionListener(DownloadAllActionListener.class, DOWNLOAD_ALL_DATA) ;
    adminRole_ = hasRole("admin") ;    
  }
  
  static public class ImportUserActionListener extends ExoActionListener  {
    public ImportUserActionListener() {
      addInterceptor(new CheckRoleInterceptor("admin")) ;  
    }
    
    public void execute(ExoActionEvent event) throws Exception {
      UIBackupUserData component = (UIBackupUserData)event.getComponent() ;
      String dataName = event.getParameter(OBJECTID) ;
      int idx = dataName.lastIndexOf('.') ;
      String userName = dataName.substring(0, idx) ;
      component.getBackupService().importUserData(userName) ;
    }
  }
  
  static public class ImportAllActionListener extends ExoActionListener  {
    public ImportAllActionListener() {
      addInterceptor(new CheckRoleInterceptor("admin")) ;  
    }
    
    public void execute(ExoActionEvent event) throws Exception {
      UIBackupUserData component = (UIBackupUserData)event.getComponent() ;
      component.getBackupService().importUserData() ;
    }
  }
  static public class DownloadAllActionListener extends ExoActionListener  {
    public DownloadAllActionListener() {
      addInterceptor(new CheckRoleInterceptor("admin")) ;  
    }
    
    public void execute(ExoActionEvent event) throws Exception {     
      UIBackupUserData u = (UIBackupUserData)event.getComponent();
      ZipUtil zip = new ZipUtil();
      Metadata mdata = new Metadata( "", "package users data", "multi-users") ;
      ByteArrayOutputStream byteOutput = new ByteArrayOutputStream();
      ZipOutputStream zipOutput =
        zip.addToArchive(new File( u.getBackupService().getUserDataDirectory()), byteOutput, false); 
      zip.addToArchive(zipOutput, (new XMLObject( mdata)).toByteArray(), "metadata.xml");
      zipOutput.close();
      ByteArrayInputStream byteInput = new ByteArrayInputStream(byteOutput.toByteArray());
      DownloadService dservice =
        ( DownloadService) PortalContainer.getComponent( DownloadService.class);
      DownloadResource dresource = new InputStreamDownloadResource(byteInput, "application/zip") ;
      dresource.setDownloadName("users.zip");
      String downloadId = dservice.addDownloadResource(dresource) ;
      String downloadLink = dservice.getDownloadLink(downloadId) ;
      FacesContext ctx = u.getFacesContext();             
      HttpServletResponse response =(HttpServletResponse)ctx.getExternalContext().getResponse();      
      response.sendRedirect(downloadLink);
      ctx.responseComplete();     
    }
  }
}