/**
 * Copyright 2001-2003 The eXo platform SARL All rights reserved.
 * Please look at license.txt in info directory for more license detail. 
 */
package org.exoplatform.portlets.management.registry.component;

import java.util.*;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.CheckRoleInterceptor;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.faces.component.model.PortletCategoryData;
import org.exoplatform.services.portletcontainer.monitor.PortletContainerMonitor;
import org.exoplatform.services.portletregistery.Portlet;
import org.exoplatform.services.portletregistery.PortletCategory;
import org.exoplatform.services.portletregistery.PortletRegisteryService;
/**
 * Created by Tuan Nguyen
 * Date: 11 macr 2005
 */
public class UIPortletCategories extends UIExoCommand {
  
  public static final String SELECT_CATEGORY = "selecCategory";
  public static final String EDIT_CATEGORY = "editCategory";
  public static final String DELETE_CATEGORY = "deleteCategory";
  public static final String ADD_PORTLET = "addPortlet";
  public static final String EDIT_PORTLET = "editPortlet";
  public static final String DELETE_PORTLET = "deletePortlet";
  public static final String EDIT_ROLE = "editRole";
  
  public static final String ADD_CATEGORY = "addCategory";
  public static final String IMPORT_PORTLETS = "importPortlets";
  public static final String REMOVE_PORTLETS = "removePortlets";
  
  private static Parameter[] SELECT_CATEGORY_PARAMS = {new Parameter(ACTION , SELECT_CATEGORY)};
  private static Parameter[] EDIT_CATEGORY_PARAMS = {new Parameter(ACTION , EDIT_CATEGORY)};
  private static Parameter[] DELETE_CATEGORY_PARAMS = {new Parameter(ACTION , DELETE_CATEGORY)};
  private static Parameter[] ADD_PORTLET_PARAMS = {new Parameter(ACTION , ADD_PORTLET)};
  
  private static Parameter[] EDIT_PORTLET_PARAMS = {new Parameter(ACTION , EDIT_PORTLET)};
  private static Parameter[] DELETE_PORTLET_PARAMS = {new Parameter(ACTION , DELETE_PORTLET)};
  private static Parameter[] EDIT_ROLE_PARAMS = {new Parameter(ACTION , EDIT_ROLE)};
  
  private static Parameter[] ADD_CATEGORY_PARAMS = {new Parameter(ACTION , ADD_CATEGORY)};
  private static Parameter[] IMPORT_PORTLETS_PARAMS = {new Parameter(ACTION , IMPORT_PORTLETS)};
  private static Parameter[] REMOVE_PORTLETS_PARAMS = {new Parameter(ACTION , REMOVE_PORTLETS)};

  
  protected Map categories_ = new LinkedHashMap() ;

  public UIPortletCategories() throws Exception {
    setRendererType("VelocityRenderer");
    setId("UIPortletSelecttor");
    addActionListener(SelectCategoryActionListener.class, SELECT_CATEGORY) ;
    addActionListener(DeleteCategoryActionListener.class, DELETE_CATEGORY) ;
    addActionListener(EditCategoryActionListener.class, EDIT_CATEGORY) ;
    addActionListener(AddPortletActionListener.class, ADD_PORTLET) ;
    addActionListener(AddCategoryActionListener.class, ADD_CATEGORY) ;
    addActionListener(EditPortletActionListener.class, EDIT_PORTLET) ;
    addActionListener(DeletePortletActionListener.class, DELETE_PORTLET) ;
    addActionListener(EditRoleActionListener.class, EDIT_ROLE) ;
    addActionListener(ImportActionListener.class, IMPORT_PORTLETS);
    addActionListener(CleanAllActionListener.class, REMOVE_PORTLETS);
    refresh() ;
  }

  public void refresh() throws Exception {
  	categories_.clear() ;
    PortletRegisteryService service =
      (PortletRegisteryService) PortalContainer.getComponent(PortletRegisteryService.class) ;
    List list = service.getPortletCategories() ;
    Collections.sort(list, new PortletCategoryComparator()) ;
    PortletComparator portletComparator = new PortletComparator() ;
    for(int i =0; i < list.size(); i++) {
    	PortletCategory portletCategory = (PortletCategory) list.get(i);
    	List portlets = service.getPortlets(portletCategory.getId()) ;
      Collections.sort(portlets, portletComparator) ;
    	PortletCategoryData data = new PortletCategoryData(portletCategory, portlets) ;
      categories_.put(portletCategory.getId(), data) ;
    }
    if(categories_.size() > 0) {
    	PortletCategoryData select = 
    	 	(PortletCategoryData)categories_.values().iterator().next() ;
    	select.setSelect(true) ;
    }
  }
  
  public Parameter[]  getSelectCategoryParams() { return SELECT_CATEGORY_PARAMS ; }
  public Parameter[]  getEditCategoryParams() { return EDIT_CATEGORY_PARAMS ; }
  public Parameter[]  getDeleteCategoryParams() { return DELETE_CATEGORY_PARAMS ; }
  public Parameter[]  getAddPortletParams() { return ADD_PORTLET_PARAMS ; }
  
  public Parameter[]  getEditPortletParams() { return EDIT_PORTLET_PARAMS ; }
  public Parameter[]  getDeletePortletParams() { return DELETE_PORTLET_PARAMS ; }
  public Parameter[]  getEditRoleParams() { return EDIT_ROLE_PARAMS ; }
  
  public Parameter[]  getAddCategoryParams() { return ADD_CATEGORY_PARAMS ; }
  public Parameter[]  getImportPortletsParams() { return IMPORT_PORTLETS_PARAMS ; }
  public Parameter[]  getRemovePortletsParams() { return REMOVE_PORTLETS_PARAMS ; }
  
  public Map getPortletCaterories() { return categories_; }
  
  public boolean hasCategory() {
    if(categories_.isEmpty())  return false;
    return true;
  }
  
  private Portlet findPortlet(String id) {
    Iterator catIterator = categories_.values().iterator() ;
    while(catIterator.hasNext()) {
      PortletCategoryData cat = (PortletCategoryData) catIterator.next() ;
      Iterator portletIterator = cat.getPortlets().iterator() ;
      while(portletIterator.hasNext()) {
        Portlet portlet = (Portlet) portletIterator.next() ;
        if(id.equals(portlet.getId())) return portlet ;
      }
    }
    return null ;
  }
  
  static public class SelectCategoryActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
    	UIPortletCategories uiSelector = (UIPortletCategories) event.getComponent() ;
    	String categoryId = event.getParameter(OBJECTID) ;
    	PortletCategoryData portletCategoryData = 
    		(PortletCategoryData) uiSelector.categories_.get(categoryId) ;
    	portletCategoryData.setSelect(!portletCategoryData.isSelect()) ;
    }
  }
  
  static public class DeleteCategoryActionListener extends ExoActionListener {
    public DeleteCategoryActionListener() {
      addInterceptor(new CheckRoleInterceptor("admin")) ;  
    }
    
    public void execute(ExoActionEvent event) throws Exception {
      UIPortletCategories uiCategories= (UIPortletCategories) event.getComponent() ;
      String categoryId = event.getParameter(OBJECTID) ;
      PortletRegisteryService service =
        (PortletRegisteryService) PortalContainer.getComponent(PortletRegisteryService.class) ;
      service.removePortletCategory(categoryId);
      uiCategories.categories_.remove(categoryId) ;
    }
  }
  
  static public class AddPortletActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIPortletCategories uiCategories = (UIPortletCategories) event.getComponent() ;
      String categoryId = event.getParameter(OBJECTID) ;
      PortletCategoryData categoryData = 
        (PortletCategoryData) uiCategories.categories_.get(categoryId) ;
      UIAvailablePortlet ui =
        (UIAvailablePortlet) uiCategories.getSibling(UIAvailablePortlet.class) ;
      ui.setPortletCategoryData(categoryData) ;
      uiCategories.setRenderedSibling(UIAvailablePortlet.class) ;
    }
  }
  
  public static class AddCategoryActionListener extends ExoActionListener {
    public AddCategoryActionListener() {
      addInterceptor(new CheckRoleInterceptor("admin")) ;  
    }
    
    public void execute(ExoActionEvent event) throws Exception {
      UIPortletCategories uiCategories= (UIPortletCategories) event.getComponent() ;
      UICategoryForm uiForm = (UICategoryForm)uiCategories.getSibling(UICategoryForm.class) ;
      uiForm.setPortletCategoryData(null) ;
      uiCategories.setRenderedSibling(UICategoryForm.class);
    }
  }
  
  public static class EditCategoryActionListener extends ExoActionListener {
    public EditCategoryActionListener() {
      addInterceptor(new CheckRoleInterceptor("admin")) ;  
    }
    
    public void execute(ExoActionEvent event) throws Exception {
      UIPortletCategories uiCategories= (UIPortletCategories) event.getComponent() ;
      String categoryId = event.getParameter(OBJECTID) ;
      PortletCategoryData portletCategoryData = 
        (PortletCategoryData) uiCategories.categories_.get(categoryId) ;
      UICategoryForm uiForm = (UICategoryForm)uiCategories.getSibling(UICategoryForm.class) ;
      uiForm.setPortletCategoryData(portletCategoryData) ;
      uiCategories.setRenderedSibling(UICategoryForm.class);
    }
  }
  
  public static class EditPortletActionListener extends ExoActionListener {
    public EditPortletActionListener() {
      addInterceptor(new CheckRoleInterceptor("admin")) ;  
    }
    
    public void execute(ExoActionEvent event) throws Exception {
      UIPortletCategories uiCategories= (UIPortletCategories) event.getComponent() ;
      String portletId = event.getParameter(OBJECTID) ;
      Portlet portlet = uiCategories.findPortlet(portletId) ;
      UIPortletForm uiForm = (UIPortletForm)uiCategories.getSibling(UIPortletForm.class) ;
      uiForm.setPortletData(portlet) ;
      uiCategories.setRenderedSibling(UIPortletForm.class);
    }
  }
  
  public static class EditRoleActionListener extends ExoActionListener {
    public EditRoleActionListener() {
      addInterceptor(new CheckRoleInterceptor("admin")) ;  
    }
    
    public void execute(ExoActionEvent event) throws Exception {
      UIPortletCategories uiCategories= (UIPortletCategories) event.getComponent() ;
      String portletId = event.getParameter(OBJECTID) ;
      Portlet portlet = uiCategories.findPortlet(portletId) ;
      UIPortletPermission uiForm = (UIPortletPermission)uiCategories.getSibling(UIPortletPermission.class) ;
      uiForm.setCurrentPortlet(portlet) ;
      uiCategories.setRenderedSibling(UIPortletPermission.class);
    }
  }
  
  static public class DeletePortletActionListener extends ExoActionListener {
    public DeletePortletActionListener() {
      addInterceptor(new CheckRoleInterceptor("admin")) ;  
    }
    
    public void execute(ExoActionEvent event) throws Exception {
      UIPortletCategories uiCategories= (UIPortletCategories) event.getComponent() ;
      String portletId = event.getParameter(OBJECTID) ;
      PortletRegisteryService service =
        (PortletRegisteryService) PortalContainer.getComponent(PortletRegisteryService.class) ;
      service.removePortlet(portletId);
      Iterator catIterator = uiCategories.categories_.values().iterator() ;
      while(catIterator.hasNext()) {
        PortletCategoryData cat = (PortletCategoryData) catIterator.next() ;
        Iterator portletIterator = cat.getPortlets().iterator() ;
        while(portletIterator.hasNext()) {
          Portlet portlet = (Portlet) portletIterator.next() ;
          if(portletId.equals(portlet.getId())) {
            portletIterator.remove() ;
            return ;
          }
        }
      }
    }
  }
  
  static public class ImportActionListener extends ExoActionListener {
    public ImportActionListener() {
      addInterceptor(new CheckRoleInterceptor("admin")) ;  
    }
    
    public void execute(ExoActionEvent event) throws Exception {
      UIPortletCategories uiCategories =  (UIPortletCategories) event.getComponent();
      PortletContainerMonitor monitor =
        ( PortletContainerMonitor) PortalContainer.getComponent( PortletContainerMonitor.class) ;
      Collection portletDatas = monitor.getPortletRuntimeDataMap().values();
      PortletRegisteryService service =
        (PortletRegisteryService) PortalContainer.getComponent(PortletRegisteryService.class) ;
      service.importPortlets(portletDatas);
      uiCategories.refresh();
    }
  }
  
  static public class CleanAllActionListener extends ExoActionListener {
    public CleanAllActionListener() {
      addInterceptor(new CheckRoleInterceptor("admin")) ;  
    }
    
    public void execute(ExoActionEvent event) throws Exception {
      UIPortletCategories uiCategories =  (UIPortletCategories) event.getComponent();
      PortletRegisteryService service =
        (PortletRegisteryService) PortalContainer.getComponent(PortletRegisteryService.class) ;
      service.clearRepository();
      uiCategories.refresh();
    }
  }
  
  static class PortletCategoryComparator implements Comparator {
    public int compare(Object arg0, Object arg1) {
      PortletCategory cat1 = (PortletCategory) arg0 ;
      PortletCategory cat2 = (PortletCategory) arg1 ;
      return cat1.getPortletCategoryName().compareTo(cat2.getPortletCategoryName()) ;
    }
  }
  
  static class PortletComparator implements Comparator {
    public int compare(Object arg0, Object arg1) {
      Portlet p1 = (Portlet) arg0 ;
      Portlet p2 = (Portlet) arg1 ;
      return p1.getPortletName().compareTo(p2.getPortletName()) ;
    }
  }
}