package org.exoplatform.portlets.monitor.component;

import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.container.PortalContainer ;
import org.exoplatform.services.chart.*;

import java.awt.Color;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
/**
 * wen, Aug 10, 2005 @ 13:07:02 
 * @author: Hung Nguyen
 * @email:   nqhungvn@users.sourceforge.net
 * @version: $Id: UIMultiChart.java,v 1.14 2005/10/15 13:07:02 nqhungvn Exp $
 */
public class UIUserActivitiesLogChart extends UIExoCommand {
  
  final public static Parameter[] BACK_PARAMS = {new Parameter(ACTION , CANCEL_ACTION) } ;
  private List listChartLinkIdParam_ ;
  private String portal_ ;
  private boolean msgError_ = false ; 
  public UIUserActivitiesLogChart() throws Exception {
    PortalContainer pcontainer =  PortalContainer.getInstance() ;
    portal_ = pcontainer.getPortalContainerInfo().getContainerName() ;    
    setId("UIUserActivitiesChart") ;
    listChartLinkIdParam_ = new ArrayList() ;
    setRendererType("VelocityRenderer") ;
    getChildren().add(new UIUserLogChartFilter());    
    addActionListener(BackActionListener.class, CANCEL_ACTION) ;
    UIStatisticsUserActionLog uiUserLog =  new UIStatisticsUserActionLog() ;
    Collection logList = uiUserLog.getChartData();
    updateUserLogDataSet(logList, uiUserLog.getChartStyle(),uiUserLog.getMaxmin()) ;    
  }
  public Parameter[] getBackParams() { return BACK_PARAMS ; }
  public String getPortalName() { return portal_ ; }
  public List getChartLinkIds(){       
    return listChartLinkIdParam_ ;     
  }
  
  public void updateUserLogDataSet(Collection chartCollection, List style, String addition) throws Exception{
    PortalContainer pcontainer =  PortalContainer.getInstance() ;
    ChartService chartService = 
      (ChartService) pcontainer.getComponentInstanceOfType(ChartService.class) ;
    portal_ = pcontainer.getPortalContainerInfo().getContainerName() ; 
    listChartLinkIdParam_ = new ArrayList() ;
    int j = 0 ;
    ArrayList dataDuration ;
    ArrayList dataCounter ;
    Data data ;
    Chart chart ;
    SelectItem items ;
    String chartLinkId ;
    String title ;
    ChartParameter chartLinkParam ;
    int width = 600 ;
    int height = 400 ;
    String msg = null; 
    Iterator<List> it = chartCollection.iterator() ;
    while (it.hasNext()){
      dataCounter = new ArrayList() ;
      dataDuration = new ArrayList() ;
      List dataList = it.next() ;
      int size = dataList.size() ;
      if (addition.equals("max") || addition.equals("min"))
        size = 1 ;
      for (int i=0; i < size ; i++) {
        Object[] row = (Object[])dataList.get(i);
        String name     = (String)row[0] ;
        Long count   = (Long)row[1] ;
        Long duration  = (Long)row[2] ;
        data = createData(duration.longValue()/60000 , "Duration", name) ;
        dataDuration.add(data) ;
        data = createData(count.longValue() , "Hits Counter", name) ;
        data.setDisplayColor(Color.BLUE) ;
        dataCounter.add(data) ;
        if( i >= 25) {
          msgError_ = true ;          
          break ; 
        }
      } 
      items = (SelectItem)style.get(j) ;
      //create Duration chart
      title = "eXoplatform User Access Duration Chart" ;
      chart = createChart(width, height, title, "Duration(min)", items.display_, dataDuration);      
      chartLinkId = chartService.creatDownloadableChartLink(chart) ;      
      chartLinkParam = new ChartParameter(chartLinkId) ;
      listChartLinkIdParam_.add(chartLinkParam) ;
      
      //create hits chart
      title = "eXoplatform User Hits Counter Chart" ;
      chart = createChart(width, height, title, "Hits", items.display_, dataCounter);      
      chartLinkId = chartService.creatDownloadableChartLink(chart) ;      
      chartLinkParam = new ChartParameter(chartLinkId) ;
      listChartLinkIdParam_.add(chartLinkParam) ;     
      j++ ;
    }    
  }
  public boolean getMsg(){ return msgError_ ; }
  private Data createData(double value, String serie, String type){
    Data data = new Data() ;
    data.setValue(value) ;
    data.setSeries(serie) ;
    data.setType(type) ;
    return data ;
  }
  private Chart createChart(int width, int height, String title, String valueLabel, String categoryLabel, ArrayList dataset) {
    Chart chart = new Chart();
    chart.setChartTitle(title);
    chart.setChartType(Chart.COLUMN_CHART);
    chart.setIs3DChart(false);
    chart.setChartWidth(width);
    chart.setChartHeight(height);
    chart.setCategoryLabel(categoryLabel);
    chart.setValueLabel(valueLabel);
    chart.setIsHorizontal(true);
    chart.setIsDisplayItemLabel(true) ;
    chart.setChartData(dataset);
    return chart ;
  }
  static public class BackActionListener extends ExoActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UIUserActivitiesLogChart uiMultiChart = (UIUserActivitiesLogChart) event.getComponent() ;
      uiMultiChart.setRenderedSibling(uiMultiChart.getSibling(UIStatisticsUserActionLog.class).getClass()) ;
    }
  }  
  static public class ChartParameter {
    private String chartLinkId_ ;    
    public ChartParameter(String id){
      chartLinkId_ = id ;
    }
    public String getChartLinkId(){ return chartLinkId_ ; }
  }
}
