/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.portal.impl;

import org.apache.commons.logging.Log;
import org.exoplatform.container.SessionContainer;
import org.exoplatform.services.log.LogService;
import org.exoplatform.services.portal.ExoPortletPreferencesPersister;
import org.exoplatform.services.portal.model.Container;
import org.exoplatform.services.portal.model.Page;
import org.exoplatform.services.portal.model.PortalConfig;
import org.exoplatform.services.portal.model.Portlet;
import org.exoplatform.services.portal.model.PortletPreferences;
import org.exoplatform.services.portletcontainer.pci.ExoWindowID;
import org.exoplatform.services.portletcontainer.pci.WindowID;
import org.exoplatform.services.portletcontainer.pci.model.ExoPortletPreferences;
import org.exoplatform.services.portletcontainer.persistence.PortletPreferencesPersister;
/**
 * Jul 16, 2004 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: PortletPreferencesPersisterImpl.java,v 1.4 2004/08/11 02:22:16 tuan08 Exp $
 */
public class PortletPreferencesPersisterImpl 
  implements  PortletPreferencesPersister, ExoPortletPreferencesPersister {
  
  private transient PortalConfigServiceImpl service_ ;
  private transient Log log_;

	public PortletPreferencesPersisterImpl(PortalConfigServiceImpl impl, LogService lservice) {
		service_ = impl ;
    log_ = lservice.getLog(getClass()); 
  }

	public ExoPortletPreferences getPortletPreferences(WindowID windowID) throws Exception {
    SessionContainer scontainer =  SessionContainer.getInstance() ;
		Portlet portlet = findPortlet(windowID, scontainer.getRemoteUser()) ;
		if(portlet != null && portlet.getPortletPreferences() != null) {
			return portlet.getPortletPreferences().toExoPortletPreferences() ;
		}
		return null  ;
	}

	public void savePortletPreferences(WindowID windowID, ExoPortletPreferences exoPref) throws Exception {
		ExoWindowID exoWindowID = (ExoWindowID) windowID ;
		String configurationSource = exoWindowID.getConfigurationSource() ;
    Container layout = null ;
    PortalConfig config = null ;
    SessionContainer scontainer =  SessionContainer.getInstance() ;
    if(ExoWindowID.DEFAULT_PORTAL_CONFIG.equals(configurationSource)) {
      config = service_.getPortalConfig(windowID.getOwner(), scontainer.getRemoteUser()) ;
      layout = config.getLayout() ;
    } else if(ExoWindowID.MOBILE_PORTAL_CONFIG.equals(configurationSource)) {
      config = service_.getPortalConfig(windowID.getOwner(), scontainer.getRemoteUser()) ;
      layout = config.getMobilePortalLayout() ;
    } else {
      layout = service_.getPage(configurationSource, scontainer.getRemoteUser()) ;
    }
    Portlet portlet = layout.findPortletByWindowId(exoWindowID.getPersistenceId()) ;
    if(portlet == null) {
      log_.error("cannot find the portlet, portlet persitence id: " + exoWindowID.getPersistenceId()) ;
      throw new Exception("Cannot find the portlet in  " + configurationSource) ;
    }
    PortletPreferences newprefs = new PortletPreferences(exoPref) ;
    portlet.setPortletPreferences(newprefs) ;   
    newprefs.setModifier(scontainer.getRemoteUser()) ;
    if(portlet.getPortletPreferences() != null) {
      newprefs.setCreator(portlet.getPortletPreferences().getCreator()) ;
    } else {
      newprefs.setCreator(scontainer.getRemoteUser()) ;
    }
    if(config != null)  service_.savePortalConfig(config) ;
    else service_.savePage((Page)layout) ;
	}

  public PortletPreferences getPortletPreferencesData(WindowID windowID) throws Exception {
    SessionContainer scontainer =  SessionContainer.getInstance() ;
    Portlet portlet = findPortlet(windowID, scontainer.getRemoteUser()) ;
    if(portlet != null && portlet.getPortletPreferences() != null) {
      return portlet.getPortletPreferences() ;
    }
    return null  ;
  }

  public void savePortletPreferencesData(WindowID windowID, PortletPreferences prefs) throws Exception {
    // TODO Auto-generated method stub
    
  }
  
  private Portlet  findPortlet(WindowID  windowID, String remoteUser) throws Exception {
    ExoWindowID exoWindowID = (ExoWindowID) windowID ;
    String configurationSource = exoWindowID.getConfigurationSource() ;
    Container layout = null ;
    PortalConfig config = null ;
    if(ExoWindowID.DEFAULT_PORTAL_CONFIG.equals(configurationSource)) {
      config = service_.getPortalConfig(windowID.getOwner(), remoteUser) ;
      layout = config.getLayout() ;
    } else if(ExoWindowID.MOBILE_PORTAL_CONFIG.equals(configurationSource)) {
      config = service_.getPortalConfig(windowID.getOwner(), remoteUser) ;
      layout = config.getMobilePortalLayout() ;
    } else {
      layout = service_.getPage(configurationSource, remoteUser) ;
    }
    return layout.findPortletByWindowId(exoWindowID.getPersistenceId()) ;
  }
}