/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.portal.test;


import java.util.GregorianCalendar;
import java.util.List;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.services.database.HibernateService;
import org.exoplatform.services.portal.log.PortalActivitiesLogService;
import org.exoplatform.services.portal.log.PortletLogData;
import org.exoplatform.services.portal.log.Query;
import org.exoplatform.test.BasicTestCase;

/**
 * Thu, May 15, 2003 @   
 * @author: Tuan Nguyen
 * @version: $Id: TestSkinConfigService.java,v 1.7 2004/07/24 16:34:06 tuan08 Exp $
 * @since: 0.0
 * @email: tuan08@yahoo.com
 */
public class TestPortletActivitiesLogService  extends BasicTestCase {
  PortalActivitiesLogService service_ ;
  Query q ;

  public TestPortletActivitiesLogService(String name) {
    super(name);
  }

  public void setUp() throws Exception {
    service_ = (PortalActivitiesLogService) PortalContainer.getInstance().
                getComponentInstanceOfType(PortalActivitiesLogService.class) ;
    q = new Query() ;
    GregorianCalendar calendar = new GregorianCalendar() ;
    calendar.set(GregorianCalendar.HOUR_OF_DAY, 0) ;
    calendar.set(GregorianCalendar.MINUTE, 0) ;
    calendar.set(GregorianCalendar.SECOND, 0) ;
    calendar.set(GregorianCalendar.MILLISECOND, 0) ;
    q.setFromTimeSlot(calendar.getTime()) ;
    calendar.set(GregorianCalendar.HOUR_OF_DAY, 24) ;
    q.setToTimeSlot(calendar.getTime()) ;
    q.setComponentId("") ;
  }

  public void tearDown() throws Exception {
  }
  
  public void testSave() throws Exception {
    service_.logPortletRender("portlet1","app1",10, false, "render") ;
    service_.logPortletRender("portlet1","app1",20, false, "render") ;
    service_.logPortletRender("portlet2","app2",40, true, "render") ;
    service_.logPortletRender("portlet3","app3",30, false, "render") ;
    Thread.sleep(2000) ;
    
    List list = service_.getPortletLogDatas(q).getAll() ;
    
    assertEquals("expect 3 log data in db", 3, list.size()) ;
    PortletLogData  portlet1data = (PortletLogData) list.get(0) ;
    assertEquals("expect  mydata is portlet1 ", "portlet1", portlet1data.getPortletId()) ;
    assertEquals("expect  sum time access  of portlet1 is ", 30, portlet1data.getSumRenderTime()) ;
    assertEquals("expect  access counter of portlet1 is ", 2, portlet1data.getRenderCounter()) ;
    assertEquals("expect  erorr access counter of portlet1 is ", 0, portlet1data.getRenderErrorCounter()) ;
    
    PortletLogData portlet2data = (PortletLogData) list.get(1) ;
    assertEquals("expect access counter of portlet2 is ", 1, portlet2data.getRenderCounter()) ;
    assertEquals("expect error access counter of portlet2 is ", 1, portlet2data.getRenderErrorCounter()) ;
    assertEquals("expect  sum time access  of portlet2 is ", 40, portlet2data.getSumRenderTime()) ;
    
    
    service_.logPortletRender("portlet3","app3",20, false, "render") ;
    service_.logPortletRender("portlet4","app4",50, true, "render") ;
    Thread.sleep(2000) ;    
    
    resetHibernateSession() ;
    
    List nextList = service_.getPortletLogDatas(q).getAll() ;
    assertEquals("expect  4 log data in db", 4, nextList.size()) ;
    PortletLogData portlet3data = (PortletLogData) nextList.get(2) ;
    
    assertEquals("expect  mydata is portlet3 ", "portlet3", portlet3data.getPortletId()) ;
    assertEquals("expect  sum time access  of portlet3 is ", 50, portlet3data.getSumRenderTime()) ;
    assertEquals("expect  access counter of portlet3 is ", 2, portlet3data.getRenderCounter()) ;
    assertEquals("expect  erorr access counter of portlet1 is ", 0, portlet3data.getRenderErrorCounter()) ;
    
    PortletLogData portlet4data = (PortletLogData) nextList.get(3) ;
    assertEquals("expect  mydata is portlet4 ", "portlet4", portlet4data.getPortletId()) ;
    assertEquals("expect access counter of portlet4 is ", 1, portlet4data.getRenderCounter()) ;
    assertEquals("expect error access counter of portlet4 is ", 1, portlet4data.getRenderErrorCounter()) ;
    assertEquals("expect summary access time of portlet4 is ", 50, portlet4data.getSumRenderTime()) ;
  }
  
  private  void  resetHibernateSession() {
    PortalContainer pcontainer = PortalContainer.getInstance() ;
    List<HibernateService> list = 
      pcontainer.getComponentInstancesOfType(HibernateService.class) ; 
    for(HibernateService hservice : list) hservice.closeSession() ;
    PortalContainer.setInstance(null) ;  
  }
  
  public void testQueryPortletLogChartDatas() throws Exception {
       
    List l = service_.getPortletLogChartDatas(q) ;
    
    Object[] row0 = (Object[])l.get(0);
    assertEquals("expect  mydata1 appId is","app1/portlet1",/*PortletAppId*/row0[0].toString()) ;
    assertEquals("expect  access counter is",2, /*rendererCounter*/Integer.parseInt(row0[1].toString())) ;
    assertEquals("expect  error access counter is",0, /*rendererErrorCounter*/Integer.parseInt(row0[2].toString())) ;
    assertEquals("expect  sum access time is",30, /*sumTime*/Integer.parseInt(row0[3].toString())) ;
    
    Object[] row1 = (Object[])l.get(1);
    assertEquals("expect  mydata2 appId is","app2/portlet2", row1[0].toString()) ;
    assertEquals("expect  access counter is",1,Integer.parseInt(row1[1].toString())) ;
    assertEquals("expect  error access counter is",1, Integer.parseInt(row1[2].toString())) ;
    assertEquals("expect  sum access time is",40, Integer.parseInt(row1[3].toString())) ;
    
    Object[] row2 = (Object[])l.get(2);
    assertEquals("expect  mydata3 appId is","app3/portlet3", row2[0].toString()) ;
    assertEquals("expect  access counter is",2,Integer.parseInt(row2[1].toString())) ;
    assertEquals("expect  error access counter is",0, Integer.parseInt(row2[2].toString())) ;
    assertEquals("expect  sum access time is",50, Integer.parseInt(row2[3].toString())) ;

    Object[] row3 = (Object[])l.get(3);
    assertEquals("expect  mydata4 appId is","app4/portlet4", row3[0].toString()) ;
    assertEquals("expect  access counter is",1,Integer.parseInt(row3[1].toString())) ;
    assertEquals("expect  error access counter is",1, Integer.parseInt(row3[2].toString())) ;
    assertEquals("expect  sum access time is",50, Integer.parseInt(row3[3].toString())) ;
    
  }
}