package org.esupportail.cas.server.handlers.example;

import org.dom4j.Element;
import org.esupportail.cas.server.util.BasicHandler;

/**
 * This class implements a very simple handler accepting one
 * particular login/password.
 *
 * @author Pascal Aubry <pascal.aubry at univ-rennes1.fr>
 */
public final class ExampleHandler extends BasicHandler {
	
	/** the only netId that the handler will accept */
	private String login;

	/** the associated password */
	private String password;

	/**
	 * Analyse the XML configuration to set netId and password attributes (constructor).
	 * 
	 * @param handlerElement the XML element that declares the handler in the configuration file
	 * @param configDebug debugging mode of the global configuration (set by default to the handler)
	 * 
	 * @throws Exception when the handler not configured correctly
	 */
	public ExampleHandler(
			final Element handlerElement, 
			final Boolean configDebug) throws Exception {
		super(handlerElement, configDebug);
		traceBegin();

		// check that a config element is present
		checkConfigElement(true);

		// get the configuration parameters
		login = getConfigSubElementContent("login", true/*needed*/);
		trace("login = " + login);
		password = getConfigSubElementContent("password", true/*needed*/);
		trace("password = " + password);

		traceEnd();
	}
	
	/**
	 * Try to authenticate a user (compare with the local credentials).
	 *
	 * @param userLogin the user's login
	 * @param userPassword the user's password
	 *
	 * @return BasicHandler.SUCCEDED on success, 
	 * BasicHandler.FAILED_CONTINUE or BasicHandler.FAILED_STOP otherwise.
	 */	
	public int authenticate(
			final String userLogin,
			final String userPassword) {
		traceBegin();
		
		trace("Checking user's login...");
		if (userLogin.equals(login) && userPassword.equals(password)) {
			trace("Users's login matches, checking user's password...");
			if (userPassword.equals(password)) {
				trace("User's password matches.");
				traceEnd("SUCCEEDED");
				return SUCCEEDED;
			} else {
				trace("User's password does not match (no more handler should be tried).");
				traceEnd("FAILED_STOP");
				return FAILED_STOP;
			}
		} else {
			trace("User's login does not match (another handler can be tried).");
			traceEnd("FAILED_CONTINUE");
			return FAILED_CONTINUE;
		}
	}
}
