package org.esupportail.cas.server.handlers.ldap;

import org.dom4j.Element;
import org.esupportail.cas.server.util.RedundantHandler;

/**
 * This class implements an LDAP server class, which can
 * authenticate users by directly binding to an LDAP directory
 * (fastbind method). It is used by FastBindLdapHandler.
 *
 * @author Pascal Aubry <pascal.aubry at univ-rennes1.fr>
 */
public final class FastBindLdapServer extends LdapServer {
	
	/**
	 * Constructor.
	 *
	 * @param handlerDebug debugging mode of the handler
	 * @param handler      the handler the server will be used by
	 * @param serverElement the XML element that declares the server 
	 * @throws Exception Exception
	 */
	public FastBindLdapServer(
			final Boolean handlerDebug,
			final RedundantHandler handler,
			final Element serverElement) throws Exception {
		super(handlerDebug, handler, serverElement);
		traceBegin();
		traceEnd();
	}
	
	/**
	 * Try to authenticate a user (by binding to the LDAP directory).
	 *
	 * @param username the user's name
	 * @param password the user's password
	 *
	 * @return Server.AUTHENTICATE_SUCCESS, Server.AUTHENTICATE_NOAUTH
	 * or Server.AUTHENTICATE_FAILURE.
	 */	
	public int authenticate(final String username,
			final String password) {
		traceBegin();
		
		FastBindLdapHandler handler = (FastBindLdapHandler) getHandler();
		
		connectAndClose(replaceTokens(handler.getFilter(), username), password);

		switch (getConnectError()) {
			case CONNECT_SUCCESS:
				trace("Connection succeeded.");
				traceEnd("AUTHENTICATE_SUCCESS");
				return AUTHENTICATE_SUCCESS;
			case CONNECT_NOAUTH:
				trace("Connection refused.");
				traceEnd("AUTHENTICATE_NOAUTH");
				return AUTHENTICATE_NOAUTH;
			default:
				trace("Connection failure.");
			traceEnd("AUTHENTICATE_FAILURE");
			return AUTHENTICATE_FAILURE;
		}
	}
	
}

