/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package suites.ecm.user;

import condition.* ;
import suites.share.* ;
import org.exoplatform.test.web.*;
import org.exoplatform.test.web.unit.*;
import org.exoplatform.test.web.validator.*;
/**
 * Created by The eXo Platform SARL
 * Author : Dung Hoang
 *          dunghoang@exoplatform.com, jobdung@yahoo.com
 * Dec 27, 2005
 */
public class UserManageActionsSuite extends WebUnitSuite {
  static String DESC = "Go to Ecm Portlet User page, click on Info tab, click on Manage Action button and to do somethings" ;
  static String NAME = "UserManageActionsSuite" ;
  String COMMENT  = "You must run AdminViewSuite before run this suite";
  
  public UserManageActionsSuite() {    
    super(NAME, DESC) ;    
    addWebSuite(new UserAddFolderSuite());
    //-----Upload a file to created Folder
    addWebUnit(
      new ClickURLLinkUnit("UploadFileToFolder", "click on Upload button").
      setURL("&uicomponent=UIToolbar&op=upload").                  
      addValidator(new ExpectFormIdValidator("uploadForm"))      
    ) ;
    //----------Save File
    def is;
    is= Thread.currentThread().
        getContextClassLoader().
        getResourceAsStream("org/exoplatform/test/web/Util.class") ;    
    
    addWebUnit(
      new SubmitFormUnit("SaveFile", "click on save button").
      setFormId("uploadForm").   
      setFileField("upload","Unit.class", is).
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIDocumentInfo&op=changeNode&objectId=/home/users/#{client.name}/test-folder/upload")).
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIDocumentInfo&op=editDocument&objectId=upload"))
    ) ;
    //-----Select Admin tab
    addWebUnit(
      new ClickURLLinkUnit("SelectAdminTab", "click on Admin tab").
      setURL("&uicomponent=UIViewController&op=selectNode&objectId=Admin").             
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=manageVersions")).      
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=manageCategories")).
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=manageActions")).      
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=exportNode")).
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=importNode"))
    ) ;    
    //-----Manage Actions
    addWebUnit(
      new ClickURLLinkUnit("ClickOnManageActions", "click on Manage Actions button").
      setURL("&uicomponent=UIToolbar&op=manageActions").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIActionList&op=add"))      
    ) ;   
    //----------Add/Edit Actions
    addWebUnit(
      new ClickURLLinkUnit("AddAction", "click on add button").
      setURL("&uicomponent=UIActionList&op=add").   
      addValidator(new ExpectFormIdValidator("selectActionForm")).
      addValidator(new ExpectFormIdValidator("workflowForm"))            
    ) ;
    addWebUnit(
      new SubmitFormUnit("SelectActionOfType", "select action of type is exo:workflowAction").
      setFormId("selectActionForm").
      setField("actionType","exo:workflowAction").
      setField("op","change").  
      addValidator(new ExpectTextValidator("ContentValidation"))      
    ) ;
    //---------------Search Validator
    addWebUnit(
      new SubmitFormUnit("SearchValidator:", "click on search button").
      setFormId("workflowForm").   
      setField("op","selectValidator").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIExoPermissionBrowser&op=selectGroup&objectId=/admin"))      
    ) ;        
    addWebUnit(
      new ClickURLLinkUnit("SelectGroup", "select a group").
      setURL("&uicomponent=UIExoPermissionBrowser&op=selectGroup&objectId=/admin").                  
      addValidator(new ExpectFormIdValidator("workflowForm"))      
    ) ;      
    //----------Save Action
    addWebUnit(
      new SubmitFormUnit("SaveNewAction", "select action of type, fill all informations and click save button").
      setFormId("workflowForm"). 
      setField("name","action-test").
      setField("lifecycle","read").      
      setField("description","description-test").
      setField("validator","*:/admin").
      setField("destWorkspace","production").                
      setField("destPath","/jcr:system").
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIActionList&op=view&objectId=action-test"))
    ) ;      
    //Delete created Action        
    addWebUnit(
      new ClickURLLinkUnit("DeleteCreatedAction", "delete created action").
      setURL("&uicomponent=UIActionList&op=delete&objectId=action-test").         
      addValidator(new ExpectNoTextValidator("action-test"))      
    ) ;
    //Saved Session
    addWebUnit(
      new ClickURLLinkUnit("SavedSession", "click on save Session button").
      setURL("&uicomponent=MainToolbar&op=saveSession").        
      addValidator(new ExpectTextValidator("All the changes have been saved"))      
    ) ;    
    //Back to previous node
    addWebUnit(
      new ClickURLLinkUnit("BackToPreviousNode", "back to previous node").
      setURL("&uicomponent=UIActionList&op=PhaseId[1].cancel")      
    ) ;
    //Select Workspace
    addWebUnit(
      new ClickURLLinkUnit("SelectWorkspace", "select Workspace button").
      setURL("&uicomponent=MainToolbar&op=selectWorkspace").                  
      addValidator(new ExpectTextValidator("Workspaces "))      
    ) ;    
  }                 
  public java.util.List getRequiredSuites() { 
    String[] roles = ["user"] ;
    return [new NewAccountSuite(), new UserLoginSuite("#{client.name}", "#{client.name}", roles)] ;
  }  
}