/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package suites.user;

import condition.* ;
import suites.share.* ;
import suites.share.UserLoginSuite;
import suites.share.NewAccountSuite;

import org.exoplatform.test.web.*;
import org.exoplatform.test.web.unit.*;
import org.exoplatform.test.web.validator.*;
/**
 * Test all the basic actions group management
 *      1)  Add 2 new account : exo-origin and exo-inherit
 *      2)  Go to Groups tab and add a new group has name is  newgroup
 *      3)  Go to newgroup and add  exo-inherit
 *      4)  Go to community config on newgroup and add Navigation community is exo-origin and add portal community is exo-origin 
 *      5)  Click on Admin to convert to exo-origin portal
 *      6)  From exo-origin add a new node has name is TestNode
 *      7)  Click on Back to go to admin portal
 *      8)  Logout and login by account exo-inherit,then exo-inherit on newgroup inherit navigation community and portal community 
 *          of exo-origin 
 *      9)  Search exo-inherit, exo-origin 
 *      10) Check creater and modify testNode
 *      11) Remove exo-inherit on newgroup
 *      12) Remove navigation community and portal community on newgroup
 *      13) Delete accounts: exo-origin,exo-inherit and newgroup 
 *      
 *      
 * May 21, 2004
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @author: Hoa Nguyen Thi
 * @email:   hoa2501@yahoo.com
 * @version: $Id: OrganizationPortletSuite.java,v 1.3 2004/10/22 14:23:55 tuan08 Exp $
 **/
public class GroupManagementSuite extends WebUnitSuite {
  static String DESC = "Go to Group Management page and test all the basic actions at the users tab" ;
  static String NAME = "GroupManagementSuite" ;
  final static public String ORIGIN_ACCOUNT = "exo-origin" ;
  final static public String INHERIT_ACCOUNT = "exo-inherit" ;
  String COMMENT = null;  
  public GroupManagementSuite() {
    super(NAME, DESC) ;
    init("/Site Map/Management", "Site Map/User Registration") ;
  }  
  public GroupManagementSuite(String path, String reg_path) {
    super(NAME, DESC) ;
    init(path, reg_path) ;
  }  
  public void  init(path, reg_path) {
    //User Registration
    goToPage(reg_path) ;    
    //-----Create exo-origin account 
    addWebSuite(new NewAccountSuite(ORIGIN_ACCOUNT, ORIGIN_ACCOUNT)) ;
    //-----Create exo-inherit account 
    addWebSuite(new NewAccountSuite(INHERIT_ACCOUNT, INHERIT_ACCOUNT)) ;
    //Management
    goToPage(path) ;
    addWebUnit(
      new ClickURLLinkUnit("GoToGroupsTab", "Click on the groups tab").
      setURL("op=selectNode&objectId=UIGroupNode").              
      addValidator(new ExpectTextValidator("Add Group"))
    ) ;
    //-----Create a new group has name is newgroup 
    addWebUnit(
      new ClickLinkWithText("ClickAddGroup", "Go to the group form").
      setTextLink("Add Group").
      addValidator(new ExpectFormIdValidator("groupForm"))
    ) ;
    COMMENT = "You must check form some field";
    addWebUnit(
      new TODO("NotCheckFormWhenSaveGroup", COMMENT).
      setComment(COMMENT).
      setStatus(WebUnit.STATUS_INCOMPLETE)
    ) ;
    addWebUnit(
      new SubmitFormUnit("CreateNewGroup", "Create newgroup").
      setFormId("groupForm").      
      setField("groupName", "newgroup").
      setField("description", "a new group").
      setField("op", "save").
      addValidator(new ExpectLinkWithTextValidator("newgroup"))
    ) ;
    addWebUnit(
      new ClickLink("Click:newgroup", "Go to 'newgroup'").
      setTextLink("newgroup").
      addValidator(new ExpectTextValidator("/newgroup")).
      addValidator(new ExpectFormIdValidator("membershipForm"))
    ) ;     
    //-----Add exo-inherit to newgroup which has user name is invalid
    addWebUnit(
        new SubmitFormUnit("AddInheritAccInvalid", "Fill all infomations and click on save").
        setFormId("membershipForm").      
        setField("username", "nameInvalid").
        setField("membershipName", "member").
        setField("groupId", "/newgroup").
        setField("op", "save").
        addValidator(new ExpectTextValidator("Your username nameInvalid is invalid"))
      ) ;
    //-----Add exo-inherit to newgroup 
    addWebUnit(
      new SubmitFormUnit("AddInheritAccValid", "Fill all infomations and click on save").
      setFormId("membershipForm").      
      setField("username", INHERIT_ACCOUNT).
      setField("membershipName", "member").
      setField("groupId", "/newgroup").
      setField("op", "save").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIViewUserInGroup&op=delete&objectId="+INHERIT_ACCOUNT))
    ) ;
    //-----Click Community Config
    addWebUnit(
      new ClickURLLinkUnit("ClickCommConfigTab", "Click on Community Config tab'").
      setURL("&uicomponent=UIGroupDetail&op=selectNode&objectId=UICommunityNode").
      addValidator(new ExpectTextValidator("add/edit"))
    ) ;     
    COMMENT = "you must have validator to test priority";
    addWebUnit(
      new TODO("can'tTestPriority", COMMENT).
      setComment(COMMENT).
      setStatus(WebUnit.STATUS_INCOMPLETE)
    ) ;
    //----------Add Navigation Community
    addWebUnit(
      new ClickURLLinkUnit("AddEditNavCommunity", "Click on add/edit'").
      setURL("&uicomponent=UIGroupCommunityInfo&op=addEditNav").
      addValidator(new ExpectFormIdValidator("communityNavForm"))      
    ) ; 
    //---------------Click on cancel
    addWebUnit(
        new SubmitFormUnit("ClickOnCancel", "Click on cancel").
        setFormId("communityNavForm").
        setField(WebUnit.ACTION, "PhaseId[1].cancel").
        addValidator(new ExpectTextValidator("add/edit"))
      ) ;
    //----------Add Navigation Community
    addWebUnit(
        new ClickURLLinkUnit("AddEditNavCommunity", "Click on add/edit'").
        setURL("&uicomponent=UIGroupCommunityInfo&op=addEditNav").
        addValidator(new ExpectFormIdValidator("communityNavForm"))      
     ) ; 
    //---------------Add Navigation Community has portal is empty
    addWebUnit(
        new SubmitFormUnit("SubmitCommNavFormEmpty", "Submit navigation portal form").
        setFormId("communityNavForm").
        setField(WebUnit.ACTION, "save").
        setField("navigation", "").
        setField("priority", "1").
        addValidator(new ExpectTextValidator("Please enter a value in the field navigation"))
      ) ;
    //---------------Add Navigation Community has portal is invalid
    addWebUnit(
        new SubmitFormUnit("SubmitCommNavFormInvalid", "Submit navigation portal form").
        setFormId("communityNavForm").
        setField(WebUnit.ACTION, "save").
        setField("navigation", "Navigation invalid").
        setField("priority", "1").
        addValidator(new ExpectTextValidator("The system does not find the user {1}"))
      ) ;
    //---------------Add Navigation Community valid
    addWebUnit(
      new SubmitFormUnit("SubmitCommNavFormValid", "Submit navigation portal form").
      setFormId("communityNavForm").
      setField(WebUnit.ACTION, "save").
      setField("navigation", ORIGIN_ACCOUNT).
      setField("priority", "1").
      addValidator(new ExpectTextValidator("add/edit"))
    ) ;
    //----------Add Portal Community
    addWebUnit(
      new ClickURLLinkUnit("AddEditPortalCommunity", "Click on add/edit'").
      setURL("uicomponent=UIGroupCommunityInfo&op=addEditPortal").
      addValidator(new ExpectFormIdValidator("communityPortalForm"))      
    ) ;  
    //---------------Click on cancel add Portal community 
    addWebUnit(
        new SubmitFormUnit("ClickOnCancel", "Click on cancel").
        setFormId("communityPortalForm").
        setField(WebUnit.ACTION, "PhaseId[1].cancel").
        addValidator(new ExpectTextValidator("add/edit"))
      ) ;
    //----------Add Portal Community
    addWebUnit(
      new ClickURLLinkUnit("AddEditPortalCommunity", "Click on add/edit'").
      setURL("uicomponent=UIGroupCommunityInfo&op=addEditPortal").
      addValidator(new ExpectFormIdValidator("communityPortalForm"))      
    ) ;   
    //---------------Add Portal community with portal field is empty 
    addWebUnit(
        new SubmitFormUnit("SubmitCommPortalFormEmpty", "Submit portal form").
        setFormId("communityPortalForm").
        setField(WebUnit.ACTION, "save").
        setField("portal","" ).
        setField("priority", "1").
        addValidator(new ExpectTextValidator("Please enter a value in the field portal"))
      ) ;
    //---------------Add Portal community with portal field is empty 
    addWebUnit(
        new SubmitFormUnit("SubmitCommPortalFormInvalid", "Submit portal form").
        setFormId("communityPortalForm").
        setField(WebUnit.ACTION, "save").
        setField("portal","portal invalide" ).
        setField("priority", "1").
        addValidator(new ExpectTextValidator("The system does not find the user {1}"))
      ) ;
    //---------------Add portal community is valid
    addWebUnit(
      new SubmitFormUnit("SubmitCommPortalFormValid", "Submit portal form").
      setFormId("communityPortalForm").
      setField(WebUnit.ACTION, "save").
      setField("portal", ORIGIN_ACCOUNT).
      setField("priority", "1").
      addValidator(new ExpectTextValidator("add/edit"))
    ) ;
    //----------Admin
    addWebUnit(
      new ClickURLLinkUnit("AddEditNavigationMode", "Click on Admin'").
      setURL("&uicomponent=UIGroupCommunityInfo&op=adminNav").
      addValidator(new ExpectNoTextValidator("navigation community")).
      addValidator(new ExpectTextValidator(ORIGIN_ACCOUNT))      
    ) ;    
    addWebUnit( CommonUnit.CLICK_EDIT_NAVIGATION_MODE) ;
    addWebUnit(CommonUnit.CLICK_SHOW_NAV) ;   
    addWebUnit(
      new ClickURLLinkUnit("ClickAddNode(onHomePage)", "Click 'add' on homepage").
      setURL("uicomponent=UINavigationManager&op=addNode&objectId=/").
      addValidator(new ExpectFormIdValidator("pageNodeForm")).
      addValidator(new ExpectTextValidator("Add/Edit Node")) 
    ) ; 
    addWebUnit(
        new SubmitFormUnit("Click on cancel", "Click on cancel").
        setFormId("pageNodeForm").
        setField("op","PhaseId[1].cancel")
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("ClickAddNode(onHomePage)", "Click 'add' on homepage").
        setURL("uicomponent=UINavigationManager&op=addNode&objectId=/").
        addValidator(new ExpectFormIdValidator("pageNodeForm")).
        addValidator(new ExpectTextValidator("Add/Edit Node")) 
      ) ; 
    addWebUnit(
        new SubmitFormUnit("Add'TestNode'NameEmpty", "Create a test node with name is empty").
        setFormId("pageNodeForm").
        setField("name","").
        setField("displayName","TestNode").
        setField("xhtmlPageReference",ORIGIN_ACCOUNT+":/home").
        setField("op","save").
        addValidator(new ExpectTextValidator("The field \"name\" cannot be empty"))      
      ) ;
    addWebUnit(
        new SubmitFormUnit("Add'TestNode'DispNameEmpty", "Create a test node with display name is empty").
        setFormId("pageNodeForm").
        setField("name","TestNode").
        setField("displayName","").
        setField("xhtmlPageReference",ORIGIN_ACCOUNT+":/home").
        setField("op","save").
        addValidator(new ExpectTextValidator("You forgot to enter the field \"displayName\""))      
      ) ;
    addWebUnit(
        new SubmitFormUnit("Add'TestNode'PageRefEmp", "Create a test node with xhtml page reference is empty").
        setFormId("pageNodeForm").
        setField("name","TestNode").
        setField("displayName","TestNode").
        setField("xhtmlPageReference","").
        setField("op","save").
        addValidator(new ExpectTextValidator("You forgot to enter the field \"xhtmlPageReference\""))      
      ) ;
    
    addWebUnit(
      new SubmitFormUnit("Add'TestNode'valid", "Create a test node with some properties are default").
      setFormId("pageNodeForm").
      setField("name","TestNode").
      setField("displayName","TestNode").
      setField("xhtmlPageReference",ORIGIN_ACCOUNT+":/home").
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UINavigationManager&op=deleteNode&objectId=/TestNode"))      
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("SaveNavigation", "Click on Save button").
      setURL("&uicomponent=UIToolbarNav&op=saveNode")
    ) ;    
    addWebUnit(
      new ClickURLLinkUnit("ClickOnBack", "Click on back button").
      setURL("admin:returnAction=return")
    ) ;   
    //Click groups tab
    addWebUnit(
        new ClickURLLinkUnit("GoToGroupsTab", "Click on the groups tab").
        setURL("op=selectNode&objectId=UIGroupNode").              
        addValidator(new ExpectTextValidator("Add Group"))
      ) ;
    //---------------Check exo-inherit's config
    addWebUnit(
      new ClickLink("SelectPortalAdmin", "Click on portal admin link").
      setTextLink("Portal Admin")
    ) ;
    COMMENT = "You must logout and login with exo-inherit account or exo-origin,then you search exo-origin or exo-inherit";
    addWebUnit(
      new TODO("WhySeeExo-origin", COMMENT).
      setComment(COMMENT).
      setStatus(WebUnit.STATUS_INCOMPLETE)
    ) ;
    addWebUnit(
      new ContextManagementUnit(ContextManagementUnit.SAVE, "Save Context")
    );     
    String[] roles=["user"];
    addWebSuite(new UserLoginSuite(INHERIT_ACCOUNT, INHERIT_ACCOUNT, roles)) ;        
    addWebUnit(
      new ContextManagementUnit(ContextManagementUnit.RESTORE, "Restore Context")
    );
    addWebUnit(
      new SubmitFormUnit("SearchInheritAcc", "Search Inherit account").
      setFormId("searchForm").
      setField("owner",INHERIT_ACCOUNT).
      setField("op","search").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIAvailablePortal&op=admin&objectId="+INHERIT_ACCOUNT))
    ) ;
    addWebUnit(
        new SubmitFormUnit("SearchOriginAcc", "Search Origin  account").
        setFormId("searchForm").
        setField("owner",ORIGIN_ACCOUNT).
        setField("op","search").
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIAvailablePortal&op=admin&objectId="+ORIGIN_ACCOUNT))
      ) ;
    addWebUnit(
      new ClickURLLinkUnit("CheckOriginAccount", "Click on edit link").
      setURL("&uicomponent=UIAvailablePortal&op=admin&objectId="+ORIGIN_ACCOUNT).
      addValidator(new ExpectLinkWithTextValidator("TestNode"))
    ) ;
    //--------------------Check creator and modifier for TestNode
    addWebUnit( CommonUnit.CLICK_EDIT_NAVIGATION_MODE) ;
    addWebUnit(CommonUnit.CLICK_SHOW_NAV) ;
    COMMENT = "You can't see TestNode's edit button, so you can't check TestNode's creator and modifier!!!";
    addWebUnit(
      new TODO("Can'tCheckCreator&Modifier", COMMENT).
      setComment(COMMENT).
      setStatus(WebUnit.STATUS_INCOMPLETE)
    ) ;
    //---------------Back
    addWebUnit(
      new ClickURLLinkUnit("ViewNormalMode", "Click on View Mode button").
      setURL("&uicomponent=UIToolbarNav&op=changeMode&mode=normal")
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("ClickOnBack", "Click on back button").
      setURL("admin:returnAction=return")
    ) ; 
    //----------Remove navigation community on newgroup
    addWebUnit(
      new ClickLinkWithText("GoToManagement", "Click on the Management link").
      setTextLink("Management").              
      addValidator(new ExpectFormIdValidator("quickSearchForm"))
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("GoToGroupsTab", "Click on the groups tab").
      setURL("op=selectNode&objectId=UIGroupNode").              
      addValidator(new ExpectTextValidator("Add Group"))
    ) ;
    addWebUnit(
      new ClickLink("Click:newgroup", "Go to 'newgroup'").
      setTextLink("newgroup").
      addValidator(new ExpectTextValidator("/newgroup")).
      addValidator(new ExpectFormIdValidator("membershipForm"))
    ) ;   
    addWebUnit(
      new ClickURLLinkUnit("RemoveExo-inherit", "Click on delete button to remove exo-inherit").
      setURL("&uicomponent=UIViewUserInGroup&op=delete&objectId="+INHERIT_ACCOUNT).              
      addValidator(new ExpectNoTextValidator(INHERIT_ACCOUNT))
    ) ;    
    //---------------Recheck exo-inherit's config
    addWebUnit(
      new ClickLink("SelectPortalAdmin", "Click on portal admin link").
      setTextLink("Portal Admin")
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("RecheckInheritAcc", "Click on edit link").
      setURL("&uicomponent=UIAvailablePortal&op=admin&objectId="+INHERIT_ACCOUNT).
      addValidator(new ExpectNoTextValidator("TestNode"))
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("ClickOnBack", "Click on back button").
      setURL("admin:returnAction=return")
    ) ; 
    //-----Delete exo-origin and exo-inherit account
    addWebUnit(
      new ClickLinkWithText("GoToManagement", "Click on the Management link").
      setTextLink("Management").              
      addValidator(new ExpectFormIdValidator("quickSearchForm"))
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("DeleteOriginAccount", "Click on delete button on the same row with exo-origin").
      setURL("&uicomponent=UIListUser&op=delete&objectId="+ORIGIN_ACCOUNT).              
      addValidator(new ExpectNoTextValidator(ORIGIN_ACCOUNT))
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("DeleteInheritAccount", "Click on delete button on the same row with exo-inherit").
      setURL("&uicomponent=UIListUser&op=delete&objectId="+INHERIT_ACCOUNT).              
      addValidator(new ExpectNoTextValidator(INHERIT_ACCOUNT))
    ) ;
    //-----Remove navigation community and portal community on newgroup
    addWebUnit(
      new ClickURLLinkUnit("GoToGroupsTab", "Click on the groups tab").
      setURL("op=selectNode&objectId=UIGroupNode").              
      addValidator(new ExpectTextValidator("Add Group"))
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("SelectNewgroup", "Select newgroup").
      setURL("&uicomponent=UIGroupExplorer&op=changeGroup&objectId=/newgroup").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIGroupExplorer&op=deleteGroup&objectId=/newgroup"))
    ) ;
    //-----Click Community Config
    addWebUnit(
      new ClickURLLinkUnit("ClickCommConfigTab", "Click on Community Config tab'").
      setURL("&uicomponent=UIGroupDetail&op=selectNode&objectId=UICommunityNode").
      addValidator(new ExpectTextValidator("add/edit"))
    ) ;  
    //----------Delete Portal Community
    addWebUnit(
      new ClickURLLinkUnit("DeletePortalCommunity", "Click on delete").
      setURL("uicomponent=UIGroupCommunityInfo&op=deletePortal").
      addValidator(new ExpectTextValidator(" "))      
    ) ; 
    //----------Delete Navigation Community
    addWebUnit(
      new ClickURLLinkUnit("DeleteNavCommunity", "Click on delete").
      setURL("uicomponent=UIGroupCommunityInfo&op=deleteNav").
      addValidator(new ExpectTextValidator(" "))      
    ) ;  
    //-----Delete newgroup
    addWebUnit(
      new ClickURLLinkUnit("DeleteNewGroup", "Delete if existed newgroup").
      setURL("&uicomponent=UIGroupExplorer&op=deleteGroup&objectId=/newgroup").
      addValidator(new ExpectTextValidator("Group newgroup has been removed!"))
    ) ;
 }
 
  public java.util.List getRequiredSuites() { 
    String[] roles = ["user", "admin"] ;
    return [new UserLoginSuite(CommonUnit.NAME_EXO_ADMIN, "exo", roles)] ;
  }
}
